@@ -49,6 +49,11 @@
 #define USE_HEAP_TYPES 1
 #endif
 
+#define BASETYPE_FLAGS \
+    Py_TPFLAGS_DEFAULT | \
+    Py_TPFLAGS_BASETYPE | \
+    Py_TPFLAGS_HAVE_GC
+
 #if PY_VERSION_HEX >= 0x030c0000
 /* Add MANAGED_WEAKREF flag for Python >= 3.12, and don't define
  * the '.tp_weaklistoffset' slot.
@@ -57,11 +62,7 @@
  *      #c.PyTypeObject.tp_weaklistoffset
  */
 #define USE_EXPLICIT_WEAKREFLIST 0
-#define BASETYPE_FLAGS \
-    Py_TPFLAGS_DEFAULT | \
-    Py_TPFLAGS_BASETYPE | \
-    Py_TPFLAGS_MANAGED_WEAKREF | \
-    Py_TPFLAGS_HAVE_GC
+#define WEAKREFTYPE_FLAGS BASETYPE_FLAGS | Py_TPFLAGS_MANAGED_WEAKREF
 #else
 /* No MANAGED_WEAKREF flag for Python < 3.12, and therefore define
  * the '.tp_weaklistoffset' slot, and the member whose offset it holds.
@@ -70,10 +71,7 @@
  *      #c.PyTypeObject.tp_weaklistoffset
  */
 #define USE_EXPLICIT_WEAKREFLIST 1
-#define BASETYPE_FLAGS \
-    Py_TPFLAGS_DEFAULT | \
-    Py_TPFLAGS_BASETYPE | \
-    Py_TPFLAGS_HAVE_GC
+#define WEAKREFTYPE_FLAGS BASETYPE_FLAGS
 #endif
 
 /* Static strings, used to invoke PyObject_GetAttr (only in hot paths) */
@@ -276,12 +274,8 @@ static void
 SB_dealloc(SB* self)
 {
     PyObject_GC_UnTrack((PyObject*)self);
+    PyObject_ClearWeakRefs(OBJECT(self));
     PyTypeObject* tp = Py_TYPE(self);
-#if USE_EXPLICIT_WEAKREFLIST
-    if (self->weakreflist != NULL) {
-        PyObject_ClearWeakRefs(OBJECT(self));
-    }
-#endif
     SB_clear(self);
     tp->tp_free(OBJECT(self));
 #if USE_HEAP_TYPES
@@ -400,7 +394,7 @@ static PyMemberDef SB_members[] = {
     { "__iro__", T_OBJECT_EX, offsetof(SB, __iro__), 0, "" },
     { "__sro__", T_OBJECT_EX, offsetof(SB, __sro__), 0, "" },
 #if USE_EXPLICIT_WEAKREFLIST
-    { "__weaklistoffset__", T_OBJECT_EX, offsetof(SB, weakreflist), 0, "" },
+    { "__weaklistoffset__", T_PYSSIZET, offsetof(SB, weakreflist), READONLY, "" },
 #endif
     { NULL },
 };
@@ -419,7 +413,7 @@ static PyTypeObject SB_type_def = {
     .tp_name           = SB__name__,
     .tp_doc            = SB__doc__,
     .tp_basicsize      = sizeof(SB),
-    .tp_flags          = BASETYPE_FLAGS,
+    .tp_flags          = WEAKREFTYPE_FLAGS,
     .tp_call           = (ternaryfunc)SB__call__,
     .tp_traverse       = (traverseproc)SB_traverse,
     .tp_clear          = (inquiry)SB_clear,
@@ -450,7 +444,7 @@ static PyType_Slot SB_type_slots[] = {
 static PyType_Spec SB_type_spec = {
     .name               = SB__name__,
     .basicsize          = sizeof(SB),
-    .flags              = BASETYPE_FLAGS,
+    .flags              = WEAKREFTYPE_FLAGS,
     .slots              = SB_type_slots
 };
 
@@ -569,8 +563,7 @@ CPB_clear(CPB* self)
 {
     Py_CLEAR(self->_cls);
     Py_CLEAR(self->_implements);
-    SB_clear((SB*)self);
-    return 0;
+    return SB_clear((SB*)self);
 }
 
 static void
@@ -649,7 +642,7 @@ static PyType_Slot CPB_type_slots[] = {
     {Py_tp_clear,       CPB_clear},
     {Py_tp_dealloc,     CPB_dealloc},
     {Py_tp_members,     CPB_members},
-    /* tp_base cannot be set as a stot -- pass to PyType_FromModuleAndSpec */
+    /* tp_base cannot be set as a slot -- pass to PyType_FromModuleAndSpec */
     {0,                 NULL}
 };
 
